@@ -10,21 +10,21 @@ import logging, platform, multiprocessing
 from tempfiles import try_delete
 
 # On Windows python suffers from a particularly nasty bug if python is spawning new processes while python itself is spawned from some other non-console process.
-# Use a custom replacement for Popen on Windows to avoid the "WindowsError: [Error 6] The handle is invalid" errors when emcc is driven through cmake or mingw32-make. 
+# Use a custom replacement for Popen on Windows to avoid the "WindowsError: [Error 6] The handle is invalid" errors when emcc is driven through cmake or mingw32-make.
 # See http://bugs.python.org/issue3905
 class WindowsPopen:
-  def __init__(self, args, bufsize=0, executable=None, stdin=None, stdout=None, stderr=None, preexec_fn=None, close_fds=False, 
+  def __init__(self, args, bufsize=0, executable=None, stdin=None, stdout=None, stderr=None, preexec_fn=None, close_fds=False,
                shell=False, cwd=None, env=None, universal_newlines=False, startupinfo=None, creationflags=0):
     self.stdin = stdin
     self.stdout = stdout
     self.stderr = stderr
-    
+
     # (stdin, stdout, stderr) store what the caller originally wanted to be done with the streams.
     # (stdin_, stdout_, stderr_) will store the fixed set of streams that workaround the bug.
     self.stdin_ = stdin
     self.stdout_ = stdout
     self.stderr_ = stderr
-    
+
     # If the caller wants one of these PIPEd, we must PIPE them all to avoid the 'handle is invalid' bug.
     if self.stdin_ == PIPE or self.stdout_ == PIPE or self.stderr_ == PIPE:
       if self.stdin_ == None:
@@ -39,7 +39,7 @@ class WindowsPopen:
     if len(args) >= 2 and args[1].endswith("emscripten.py"):
       response_filename = response_file.create_response_file(args[2:], TEMP_DIR)
       args = args[0:2] + ['@' + response_filename]
-      
+
     try:
       # Call the process with fixed streams.
       self.process = subprocess.Popen(args, bufsize, executable, self.stdin_, self.stdout_, self.stderr_, preexec_fn, close_fds, shell, cwd, env, universal_newlines, startupinfo, creationflags)
@@ -73,7 +73,7 @@ class WindowsPopen:
 
   def kill(self):
     return self.process.kill()
-  
+
 __rootpath__ = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
 def path_from_root(*pathelems):
   return os.path.join(__rootpath__, *pathelems)
@@ -168,10 +168,11 @@ def add_coloring_to_emit_ansi(fn):
 WINDOWS = sys.platform.startswith('win')
 OSX = sys.platform == 'darwin'
 
-if WINDOWS:
-  logging.StreamHandler.emit = add_coloring_to_emit_windows(logging.StreamHandler.emit)
-else:
-  logging.StreamHandler.emit = add_coloring_to_emit_ansi(logging.StreamHandler.emit)
+if sys.stderr.isatty():
+  if WINDOWS:
+    logging.StreamHandler.emit = add_coloring_to_emit_windows(logging.StreamHandler.emit)
+  else:
+    logging.StreamHandler.emit = add_coloring_to_emit_ansi(logging.StreamHandler.emit)
 
 # Emscripten configuration is done through the --em-config command line option or
 # the EM_CONFIG environment variable. If the specified string value contains newline
@@ -224,6 +225,9 @@ else:
     config_file = config_file.replace('\'{{{ EMSCRIPTEN_ROOT }}}\'', repr(__rootpath__))
     llvm_root = os.path.dirname(find_executable('llvm-dis') or '/usr/bin/llvm-dis')
     config_file = config_file.replace('\'{{{ LLVM_ROOT }}}\'', repr(llvm_root))
+    binaryen_root = os.path.dirname(find_executable('asm2wasm') or '/usr/bin/asm2wasm')
+    config_file = config_file.replace('\'{{{ BINARYEN_ROOT }}}\'', repr(binaryen_root))
+
     node = find_executable('nodejs') or find_executable('node') or 'node'
     config_file = config_file.replace('\'{{{ NODE }}}\'', repr(node))
     if WINDOWS:
@@ -262,7 +266,7 @@ except Exception, e:
   logging.error('Error in evaluating %s (at %s): %s, text: %s' % (EM_CONFIG, CONFIG_FILE, str(e), config_text))
   sys.exit(1)
 
-# Returns a suggestion where current .emscripten config file might be located (if EM_CONFIG env. var is used 
+# Returns a suggestion where current .emscripten config file might be located (if EM_CONFIG env. var is used
 # without a file, this hints to "default" location at ~/.emscripten)
 def hint_config_file_location():
   if CONFIG_FILE: return CONFIG_FILE
@@ -300,7 +304,7 @@ except:
 
 # Install our replacement Popen handler if we are running on Windows to avoid python spawn process function.
 # nb. This is by default disabled since it has the adverse effect of buffering up all logging messages, which makes
-# builds look unresponsive (messages are printed only after the whole build finishes). Whether this workaround is needed 
+# builds look unresponsive (messages are printed only after the whole build finishes). Whether this workaround is needed
 # seems to depend on how the host application that invokes emcc has set up its stdout and stderr.
 if EM_POPEN_WORKAROUND and os.name == 'nt':
   logging.debug('Installing Popen workaround handler to avoid bug http://bugs.python.org/issue3905')
@@ -315,7 +319,7 @@ EM_BUILD_VERBOSE_LEVEL = int(os.getenv('EM_BUILD_VERBOSE')) if os.getenv('EM_BUI
 
 # Expectations
 
-EXPECTED_LLVM_VERSION = (3, 8)
+EXPECTED_LLVM_VERSION = (3, 9)
 
 actual_clang_version = None
 
@@ -358,43 +362,66 @@ def get_fastcomp_src_dir():
       d = os.path.dirname(d)
   return None
 
-def check_fastcomp():
+def get_llc_targets():
   try:
     llc_version_info = Popen([LLVM_COMPILER, '--version'], stdout=PIPE).communicate()[0]
     pre, targets = llc_version_info.split('Registered Targets:')
-    if 'js' not in targets or 'JavaScript (asm.js, emscripten) backend' not in targets:
-      logging.critical('fastcomp in use, but LLVM has not been built with the JavaScript backend as a target, llc reports:')
-      print >> sys.stderr, '==========================================================================='
-      print >> sys.stderr, llc_version_info,
-      print >> sys.stderr, '==========================================================================='
-      logging.critical('you can fall back to the older (pre-fastcomp) compiler core, although that is not recommended, see http://kripken.github.io/emscripten-site/docs/building_from_source/LLVM-Backend.html')
-      return False
+    return targets
+  except Exception, e:
+    return '(no targets could be identified: ' + str(e) + ')'
 
-    # check repo versions
-    d = get_fastcomp_src_dir()
-    shown_repo_version_error = False
-    if d is not None:
-      llvm_version = get_emscripten_version(os.path.join(d, 'emscripten-version.txt'))
-      if os.path.exists(os.path.join(d, 'tools', 'clang', 'emscripten-version.txt')):
-        clang_version = get_emscripten_version(os.path.join(d, 'tools', 'clang', 'emscripten-version.txt'))
-      elif os.path.exists(os.path.join(d, 'tools', 'clang')):
-        clang_version = '?' # Looks like the LLVM compiler tree has an old checkout from the time before it contained a version.txt: Should update!
-      else:
-        clang_version = llvm_version # This LLVM compiler tree does not have a tools/clang, so it's probably an out-of-source build directory. No need for separate versioning.
-      if EMSCRIPTEN_VERSION != llvm_version or EMSCRIPTEN_VERSION != clang_version:
-        logging.error('Emscripten, llvm and clang repo versions do not match, this is dangerous (%s, %s, %s)', EMSCRIPTEN_VERSION, llvm_version, clang_version)
-        logging.error('Make sure to use the same branch in each repo, and to be up-to-date on each. See http://kripken.github.io/emscripten-site/docs/building_from_source/LLVM-Backend.html')
-        shown_repo_version_error = True
+def has_asm_js_target(targets):
+  return 'js' in targets and 'JavaScript (asm.js, emscripten) backend' in targets
+
+def has_wasm_target(targets):
+  return 'wasm32' in targets and 'WebAssembly 32-bit' in targets
+
+def check_fastcomp():
+  try:
+    targets = get_llc_targets()
+    if get_llvm_target() == ASM_JS_TARGET:
+      if not has_asm_js_target(targets):
+        logging.critical('fastcomp in use, but LLVM has not been built with the JavaScript backend as a target, llc reports:')
+        print >> sys.stderr, '==========================================================================='
+        print >> sys.stderr, targets
+        print >> sys.stderr, '==========================================================================='
+        logging.critical('you can fall back to the older (pre-fastcomp) compiler core, although that is not recommended, see http://kripken.github.io/emscripten-site/docs/building_from_source/LLVM-Backend.html')
+        return False
     else:
-      logging.warning('did not see a source tree above or next to the LLVM root directory (guessing based on directory of %s), could not verify version numbers match' % LLVM_COMPILER)
+      assert get_llvm_target() == WASM_TARGET
+      if not has_wasm_target(targets):
+        logging.critical('WebAssembly set as target, but LLVM has not been built with the WebAssembly backend, llc reports:')
+        print >> sys.stderr, '==========================================================================='
+        print >> sys.stderr, targets
+        print >> sys.stderr, '==========================================================================='
+        return False
 
-    # check build versions. don't show it if the repos are wrong, user should fix that first
-    if not shown_repo_version_error:
-      clang_v = Popen([CLANG, '--version'], stdout=PIPE).communicate()[0]
-      llvm_build_version, clang_build_version = clang_v.split('(emscripten ')[1].split(')')[0].split(' : ')
-      if EMSCRIPTEN_VERSION != llvm_build_version or EMSCRIPTEN_VERSION != clang_build_version:
-        logging.error('Emscripten, llvm and clang build versions do not match, this is dangerous (%s, %s, %s)', EMSCRIPTEN_VERSION, llvm_build_version, clang_build_version)
-        logging.error('Make sure to rebuild llvm and clang after updating repos')
+    if get_llvm_target() == ASM_JS_TARGET:
+      # check repo versions
+      d = get_fastcomp_src_dir()
+      shown_repo_version_error = False
+      if d is not None:
+        llvm_version = get_emscripten_version(os.path.join(d, 'emscripten-version.txt'))
+        if os.path.exists(os.path.join(d, 'tools', 'clang', 'emscripten-version.txt')):
+          clang_version = get_emscripten_version(os.path.join(d, 'tools', 'clang', 'emscripten-version.txt'))
+        elif os.path.exists(os.path.join(d, 'tools', 'clang')):
+          clang_version = '?' # Looks like the LLVM compiler tree has an old checkout from the time before it contained a version.txt: Should update!
+        else:
+          clang_version = llvm_version # This LLVM compiler tree does not have a tools/clang, so it's probably an out-of-source build directory. No need for separate versioning.
+        if EMSCRIPTEN_VERSION != llvm_version or EMSCRIPTEN_VERSION != clang_version:
+          logging.error('Emscripten, llvm and clang repo versions do not match, this is dangerous (%s, %s, %s)', EMSCRIPTEN_VERSION, llvm_version, clang_version)
+          logging.error('Make sure to use the same branch in each repo, and to be up-to-date on each. See http://kripken.github.io/emscripten-site/docs/building_from_source/LLVM-Backend.html')
+          shown_repo_version_error = True
+      else:
+        logging.warning('did not see a source tree above or next to the LLVM root directory (guessing based on directory of %s), could not verify version numbers match' % LLVM_COMPILER)
+
+      # check build versions. don't show it if the repos are wrong, user should fix that first
+      if not shown_repo_version_error:
+        clang_v = Popen([CLANG, '--version'], stdout=PIPE).communicate()[0]
+        llvm_build_version, clang_build_version = clang_v.split('(emscripten ')[1].split(')')[0].split(' : ')
+        if EMSCRIPTEN_VERSION != llvm_build_version or EMSCRIPTEN_VERSION != clang_build_version:
+          logging.error('Emscripten, llvm and clang build versions do not match, this is dangerous (%s, %s, %s)', EMSCRIPTEN_VERSION, llvm_build_version, clang_build_version)
+          logging.error('Make sure to rebuild llvm and clang after updating repos')
 
     return True
   except Exception, e:
@@ -468,7 +495,7 @@ except Exception, e:
   EMSCRIPTEN_VERSION = 'unknown'
 
 def generate_sanity():
-  return EMSCRIPTEN_VERSION + '|' + get_llvm_target() + '|' + LLVM_ROOT + '|' + get_clang_version()
+  return EMSCRIPTEN_VERSION + '|' + LLVM_ROOT + '|' + get_clang_version() + ('_wasm' if get_llvm_target() == WASM_TARGET else '')
 
 def check_sanity(force=False):
   try:
@@ -480,13 +507,15 @@ def check_sanity(force=False):
     else:
       settings_mtime = os.stat(CONFIG_FILE).st_mtime
       sanity_file = CONFIG_FILE + '_sanity'
+      if get_llvm_target() == WASM_TARGET:
+        sanity_file += '_wasm'
       if os.path.exists(sanity_file):
         try:
           sanity_mtime = os.stat(sanity_file).st_mtime
           if sanity_mtime <= settings_mtime:
             reason = 'settings file has changed'
           else:
-            sanity_data = open(sanity_file).read()
+            sanity_data = open(sanity_file).read().rstrip('\n\r') # workaround weird bug with read() that appends new line char in some old python version
             if sanity_data != generate_sanity():
               reason = 'system change: %s vs %s' % (generate_sanity(), sanity_data)
             else:
@@ -785,57 +814,122 @@ except:
 
 # Additional compiler options
 
-# Target choice. Must be synced with src/settings.js (TARGET_*)
+# Target choice.
+ASM_JS_TARGET = 'asmjs-unknown-emscripten'
+WASM_TARGET = 'wasm32-unknown-unknown'
+
+def check_vanilla():
+  global LLVM_TARGET
+  # if the env var tells us what to do, do that
+  if 'EMCC_WASM_BACKEND' in os.environ:
+    if os.environ['EMCC_WASM_BACKEND'] != '0':
+      logging.debug('EMCC_WASM_BACKEND tells us to use wasm backend')
+      LLVM_TARGET = WASM_TARGET
+    else:
+      logging.debug('EMCC_WASM_BACKEND tells us to use asm.js backend')
+      LLVM_TARGET = ASM_JS_TARGET
+  else:
+    # if we are using vanilla LLVM, i.e. we don't have our asm.js backend, then we
+    # must use wasm (or at least try to). to know that, we have to run llc to
+    # see which backends it has. we cache this result.
+    temp_cache = cache.Cache(use_subdir=False)
+    def check_vanilla():
+      logging.debug('testing for asm.js target, because if not present (i.e. this is plain vanilla llvm, not emscripten fastcomp), we will use the wasm target instead (set EMCC_WASM_BACKEND to skip this check)')
+      targets = get_llc_targets()
+      return has_wasm_target(targets) and not has_asm_js_target(targets)
+    def get_vanilla_file():
+      saved_file = os.path.join(temp_cache.dirname, 'is_vanilla.txt')
+      open(saved_file, 'w').write(('1' if check_vanilla() else '0') + ':' + LLVM_ROOT)
+      return saved_file
+    is_vanilla_file = temp_cache.get('is_vanilla', get_vanilla_file, extension='.txt')
+    if CONFIG_FILE and os.stat(CONFIG_FILE).st_mtime > os.stat(is_vanilla_file).st_mtime:
+      logging.debug('config file changed since we checked vanilla; re-checking')
+      is_vanilla_file = temp_cache.get('is_vanilla', get_vanilla_file, extension='.txt', force=True)
+    try:
+      contents = open(is_vanilla_file).read()
+      is_vanilla, llvm_used = contents.split(':')
+      is_vanilla = int(is_vanilla)
+      if llvm_used != LLVM_ROOT:
+        logging.debug('regenerating vanilla check since other llvm')
+        temp_cache.get('is_vanilla', get_vanilla_file, extension='.txt', force=True)
+        is_vanilla = check_vanilla()
+    except Exception, e:
+      logging.debug('failed to use vanilla file, will re-check: ' + str(e))
+      is_vanilla = check_vanilla()
+    temp_cache = None
+    if is_vanilla:
+      logging.debug('check tells us to use wasm backend')
+      LLVM_TARGET = WASM_TARGET
+      os.environ['EMCC_WASM_BACKEND'] = '1'
+    else:
+      logging.debug('check tells us to use asm.js backend')
+      LLVM_TARGET = ASM_JS_TARGET
+
+check_vanilla()
+
 def get_llvm_target():
-  return 'asmjs-unknown-emscripten'
-LLVM_TARGET = get_llvm_target()
+  global LLVM_TARGET
+  assert LLVM_TARGET is not None
+  return LLVM_TARGET
 
 # COMPILER_OPTS: options passed to clang when generating bitcode for us
 try:
   COMPILER_OPTS # Can be set in EM_CONFIG, optionally
 except:
   COMPILER_OPTS = []
+
+# Set the LIBCPP ABI version to at least 2 so that we get nicely aligned string
+# data and other nice fixes.
 COMPILER_OPTS = COMPILER_OPTS + [#'-fno-threadsafe-statics', # disabled due to issue 1289
-                                 '-target', LLVM_TARGET,
+                                 '-target', get_llvm_target(),
                                  '-D__EMSCRIPTEN_major__=' + str(EMSCRIPTEN_VERSION_MAJOR),
                                  '-D__EMSCRIPTEN_minor__=' + str(EMSCRIPTEN_VERSION_MINOR),
-                                 '-D__EMSCRIPTEN_tiny__=' + str(EMSCRIPTEN_VERSION_TINY)]
+                                 '-D__EMSCRIPTEN_tiny__=' + str(EMSCRIPTEN_VERSION_TINY),
+                                 '-D_LIBCPP_ABI_VERSION=2']
+
+if LLVM_TARGET == WASM_TARGET:
+  # wasm target does not automatically define emscripten stuff, so do it here.
+  COMPILER_OPTS = COMPILER_OPTS + ['-DEMSCRIPTEN',
+                                   '-D__EMSCRIPTEN__']
 
 # Changes to default clang behavior
-if LLVM_TARGET == 'asmjs-unknown-emscripten':
-  # Implicit functions can cause horribly confusing asm.js function pointer type errors, see #2175
-  # If your codebase really needs them - very unrecommended! - you can disable the error with
-  #   -Wno-error=implicit-function-declaration
-  # or disable even a warning about it with
-  #   -Wno-implicit-function-declaration
-  COMPILER_OPTS += ['-Werror=implicit-function-declaration']
+
+# Implicit functions can cause horribly confusing function pointer type errors, see #2175
+# If your codebase really needs them - very unrecommended! - you can disable the error with
+#   -Wno-error=implicit-function-declaration
+# or disable even a warning about it with
+#   -Wno-implicit-function-declaration
+COMPILER_OPTS += ['-Werror=implicit-function-declaration']
 
 USE_EMSDK = not os.environ.get('EMMAKEN_NO_SDK')
 
 if USE_EMSDK:
   # Disable system C and C++ include directories, and add our own (using -idirafter so they are last, like system dirs, which
   # allows projects to override them)
-  C_INCLUDE_PATHS = [path_from_root('system', 'local', 'include'),
-                     path_from_root('system', 'include', 'compat'),
-                     path_from_root('system', 'include'),
-                     path_from_root('system', 'include', 'emscripten'),
-                     path_from_root('system', 'include', 'libc'),
-                     path_from_root('system', 'lib', 'libc', 'musl', 'arch', 'emscripten')
+  C_INCLUDE_PATHS = [
+    path_from_root('system', 'include', 'compat'),
+    path_from_root('system', 'include'),
+    path_from_root('system', 'include', 'emscripten'),
+    path_from_root('system', 'include', 'libc'),
+    path_from_root('system', 'lib', 'libc', 'musl', 'arch', 'emscripten'),
+    path_from_root('system', 'local', 'include')
   ]
-  
-  CXX_INCLUDE_PATHS = [path_from_root('system', 'include', 'libcxx')
+
+  CXX_INCLUDE_PATHS = [
+    path_from_root('system', 'include', 'libcxx'),
+    path_from_root('system', 'lib', 'libcxxabi', 'include')
   ]
-  
-  C_OPTS = ['-nostdinc', '-Xclang', '-nobuiltininc', '-Xclang', '-nostdsysteminc',
-  ]
-  
+
+  C_OPTS = ['-nostdinc', '-Xclang', '-nobuiltininc', '-Xclang', '-nostdsysteminc']
+
   def include_directive(paths):
     result = []
     for path in paths:
       result += ['-Xclang', '-isystem' + path]
     return result
-  
-  EMSDK_OPTS = C_OPTS + include_directive(C_INCLUDE_PATHS) + include_directive(CXX_INCLUDE_PATHS)
+
+  # libcxx include paths must be defined before libc's include paths otherwise libcxx will not build
+  EMSDK_OPTS = C_OPTS + include_directive(CXX_INCLUDE_PATHS) + include_directive(C_INCLUDE_PATHS)
 
   EMSDK_CXX_OPTS = []
   COMPILER_OPTS += EMSDK_OPTS
@@ -843,9 +937,6 @@ else:
   EMSDK_OPTS = []
   EMSDK_CXX_OPTS = []
 
-#print >> sys.stderr, 'SDK opts', ' '.join(EMSDK_OPTS)
-#print >> sys.stderr, 'Compiler opts', ' '.join(COMPILER_OPTS)
-
 # Engine tweaks
 
 try:
@@ -977,9 +1068,12 @@ class Settings2(type):
       for i in range(len(args)):
         if args[i].startswith('-O'):
           v = args[i][2]
-          if v in ['s', 'z']: v = '2'
+          shrink = 0
+          if v in ['s', 'z']:
+            v = '2'
+            shrink = 1 if v == 's' else 2
           level = eval(v)
-          self.apply_opt_level(level)
+          self.apply_opt_level(level, shrink)
       for i in range(len(args)):
         if args[i] == '-s':
           declare = re.sub(r'([\w\d]+)\s*=\s*(.+)', r'self.attrs["\1"]=\2;', args[i+1])
@@ -1001,12 +1095,14 @@ class Settings2(type):
       self.attrs = values
 
     @classmethod
-    def apply_opt_level(self, opt_level, noisy=False):
+    def apply_opt_level(self, opt_level, shrink_level=0, noisy=False):
       if opt_level >= 1:
         self.attrs['ASM_JS'] = 1
         self.attrs['ASSERTIONS'] = 0
         self.attrs['DISABLE_EXCEPTION_CATCHING'] = 1
         self.attrs['ALIASING_FUNCTION_POINTERS'] = 1
+      if shrink_level >= 2:
+        self.attrs['EVAL_CTORS'] = 1
 
     def __getattr__(self, attr):
       if attr in self.attrs:
@@ -1063,6 +1159,7 @@ class Building:
     env['CXX'] = EMXX if not WINDOWS else 'python %r' % EMXX
     env['AR'] = EMAR if not WINDOWS else 'python %r' % EMAR
     env['LD'] = EMCC if not WINDOWS else 'python %r' % EMCC
+    env['NM'] = LLVM_NM
     env['LDSHARED'] = EMCC if not WINDOWS else 'python %r' % EMCC
     env['RANLIB'] = EMRANLIB if not WINDOWS else 'python %r' % EMRANLIB
     env['EMMAKEN_COMPILER'] = Building.COMPILER
@@ -1086,24 +1183,24 @@ class Building:
     def is_exe(fpath):
       return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
 
+    if os.path.isabs(program):
+      if os.path.isfile(program): return program
+
+      if WINDOWS:
+        for suffix in ['.exe', '.cmd', '.bat']:
+          if is_exe(program + suffix): return program + suffix
+
     fpath, fname = os.path.split(program)
     if fpath:
-      if is_exe(program):
-        return program
+      if is_exe(program): return program
     else:
       for path in os.environ["PATH"].split(os.pathsep):
         path = path.strip('"')
         exe_file = os.path.join(path, program)
-        if is_exe(exe_file):
-          return exe_file
-
-        if WINDOWS and not '.' in fname:
-          if is_exe(exe_file + '.exe'):
-            return exe_file + '.exe'
-          if is_exe(exe_file + '.cmd'):
-            return exe_file + '.cmd'
-          if is_exe(exe_file + '.bat'):
-            return exe_file + '.bat'
+        if is_exe(exe_file): return exe_file
+        if WINDOWS:
+          for suffix in ['.exe', '.cmd', '.bat']:
+            if is_exe(exe_file + suffix): return exe_file + suffix
 
     return None
 
@@ -1117,7 +1214,7 @@ class Building:
     path = filter(lambda p: not os.path.exists(os.path.join(p, 'sh.exe')), path)
     env['PATH'] = ';'.join(path)
     return env
-  
+
   @staticmethod
   def handle_CMake_toolchain(args, env):
 
@@ -1155,7 +1252,7 @@ class Building:
       args, env = Building.handle_CMake_toolchain(args, env)
     else:
       # When we configure via a ./configure script, don't do config-time compilation with emcc, but instead
-      # do builds natively with Clang. This is a heuristic emulation that may or may not work. 
+      # do builds natively with Clang. This is a heuristic emulation that may or may not work.
       env['EMMAKEN_JUST_CONFIGURE'] = '1'
     try:
       if EM_BUILD_VERBOSE_LEVEL >= 3: print >> sys.stderr, 'configure: ' + str(args)
@@ -1242,7 +1339,7 @@ class Building:
         pass # Ignore exit code != 0
     def open_make_out(i, mode='r'):
       return open(os.path.join(project_dir, 'make_' + str(i)), mode)
-    
+
     def open_make_err(i, mode='r'):
       return open(os.path.join(project_dir, 'make_err' + str(i)), mode)
 
@@ -1291,7 +1388,6 @@ class Building:
         return f
       else:
         return os.path.abspath(f)
-    files = map(make_paths_absolute, files)
     # Paths of already included object files from archives.
     added_contents = set()
     # Map of archive name to list of extracted object file paths.
@@ -1299,7 +1395,7 @@ class Building:
     has_ar = False
     for f in files:
       if not f.startswith('-'):
-        has_ar = has_ar or Building.is_ar(f)
+        has_ar = has_ar or Building.is_ar(make_paths_absolute(f))
 
     # If we have only one archive or the force_archive_contents flag is set,
     # then we will add every object file we see, regardless of whether it
@@ -1383,6 +1479,7 @@ class Building:
 
     current_archive_group = None
     for f in files:
+      absolute_path_f = make_paths_absolute(f)
       if f.startswith('-'):
         if f in ['--start-group', '-(']:
           assert current_archive_group is None, 'Nested --start-group, missing --end-group?'
@@ -1402,10 +1499,10 @@ class Building:
           current_archive_group = None
         else:
           logging.debug('Ignoring unsupported link flag: %s' % f)
-      elif not Building.is_ar(f):
-        if Building.is_bitcode(f):
+      elif not Building.is_ar(absolute_path_f):
+        if Building.is_bitcode(absolute_path_f):
           if has_ar:
-            consider_object(f, force_add=True)
+            consider_object(absolute_path_f, force_add=True)
           else:
             # If there are no archives then we can simply link all valid bitcode
             # files and skip the symbol table stuff.
@@ -1413,11 +1510,11 @@ class Building:
       else:
         # Extract object files from ar archives, and link according to gnu ld semantics
         # (link in an entire .o from the archive if it supplies symbols still unresolved)
-        consider_archive(f)
+        consider_archive(absolute_path_f)
         # If we're inside a --start-group/--end-group section, add to the list
         # so we can loop back around later.
         if current_archive_group is not None:
-          current_archive_group.append(f)
+          current_archive_group.append(absolute_path_f)
     assert current_archive_group is None, '--start-group without matching --end-group'
 
     try_delete(target)
@@ -1438,16 +1535,10 @@ class Building:
 
       response_fh = open(response_file, 'w')
       for arg in actual_files:
-        # we can't put things with spaces in the response file
-        if " " in arg:
-          link_args.append(arg)
-        else:
-          response_fh.write(arg + "\n")
+        # escaped double quotes allows 'space' characters in pathname the response file can use
+        response_fh.write("\"" + arg + "\"\n")
       response_fh.close()
 
-      if len(' '.join(link_args)) > 8192:
-        logging.warning('emcc: link command line is very long, even with response file -- use paths with no spaces')
-
     if not just_calculate:
       logging.debug('emcc: llvm-linking: %s to %s', actual_files, target)
       output = Popen([LLVM_LINK] + link_args + ['-o', target], stdout=PIPE).communicate()[0]
@@ -1659,6 +1750,71 @@ class Building:
       ret = output_filename
     return ret
 
+  @staticmethod
+  def eval_ctors(js_file, mem_init_file):
+    subprocess.check_call([PYTHON, path_from_root('tools', 'ctor_evaller.py'), js_file, mem_init_file, str(Settings.TOTAL_MEMORY), str(Settings.TOTAL_STACK), str(Settings.GLOBAL_BASE)])
+
+  @staticmethod
+  def eliminate_duplicate_funcs(filename):
+    import duplicate_function_eliminator
+    duplicate_function_eliminator.eliminate_duplicate_funcs(filename)
+
+  @staticmethod
+  def calculate_reachable_functions(infile, initial_list, can_reach=True):
+    import asm_module
+    temp = configuration.get_temp_files().get('.js').name
+    Building.js_optimizer(infile, ['dumpCallGraph'], output_filename=temp, just_concat=True)
+    asm = asm_module.AsmModule(temp)
+    lines = asm.funcs_js.split('\n')
+    can_call = {}
+    for i in range(len(lines)):
+      line = lines[i]
+      if line.startswith('// REACHABLE '):
+        curr = json.loads(line[len('// REACHABLE '):])
+        func = curr[0]
+        targets = curr[2]
+        can_call[func] = set(targets)
+    # function tables too - treat a function all as a function that can call anything in it, which is effectively what it is
+    for name, funcs in asm.tables.iteritems():
+      can_call[name] = set(map(lambda x: x.strip(), funcs[1:-1].split(',')))
+    #print can_call
+    # Note: We ignore calls in from outside the asm module, so you could do emterpreted => outside => emterpreted, and we would
+    #       miss the first one there. But this is acceptable to do, because we can't save such a stack anyhow, due to the outside!
+    #print 'can call', can_call, '\n!!!\n', asm.tables, '!'
+    reachable_from = {}
+    for func, targets in can_call.iteritems():
+      for target in targets:
+        if target not in reachable_from:
+          reachable_from[target] = set()
+        reachable_from[target].add(func)
+    #print 'reachable from', reachable_from
+    to_check = initial_list[:]
+    advised = set()
+    if can_reach:
+      # find all functions that can reach the initial list
+      while len(to_check) > 0:
+        curr = to_check.pop()
+        if curr in reachable_from:
+          for reacher in reachable_from[curr]:
+            if reacher not in advised:
+              if not JS.is_dyn_call(reacher) and not JS.is_function_table(reacher): advised.add(str(reacher))
+              to_check.append(reacher)
+    else:
+      # find all functions that are reachable from the initial list, including it
+      # all tables are assumed reachable, as they can be called from dyncall from outside
+      for name, funcs in asm.tables.iteritems():
+        to_check.append(name)
+      while len(to_check) > 0:
+        curr = to_check.pop()
+        if not JS.is_function_table(curr):
+          advised.add(curr)
+        if curr in can_call:
+          for target in can_call[curr]:
+            if target not in advised:
+              advised.add(str(target))
+              to_check.append(target)
+    return { 'reachable': list(advised), 'total_funcs': len(can_call) }
+
   @staticmethod
   def closure_compiler(filename, pretty=True):
     if not check_closure_compiler():
@@ -1732,10 +1888,10 @@ class Building:
   def ensure_struct_info(info_path):
     if os.path.exists(info_path): return
     Cache.ensure()
-    
+
     import gen_struct_info
     gen_struct_info.main(['-qo', info_path, path_from_root('src/struct_info.json')])
-  
+
 # compatibility with existing emcc, etc. scripts
 Cache = cache.Cache(debug=DEBUG_CACHE)
 chunkify = cache.chunkify
@@ -1937,8 +2093,17 @@ class JS:
     s = ''.join(map(chr, s))
     s = s.replace('\\', '\\\\').replace("'", "\\'")
     s = s.replace('\n', '\\n').replace('\r', '\\r')
+    # Escape the ^Z (= 0x1a = substitute) ASCII character and all characters higher than 7-bit ASCII.
     def escape(x): return '\\x{:02x}'.format(ord(x.group()))
-    return re.sub('[\x80-\xff]', escape, s)
+    return re.sub('[\x1a\x80-\xff]', escape, s)
+
+  @staticmethod
+  def is_dyn_call(func):
+    return func.startswith('dynCall_')
+
+  @staticmethod
+  def is_function_table(name):
+    return name.startswith('FUNCTION_TABLE_')
 
 def execute(cmd, *args, **kw):
   try:
